@@ -1,10 +1,13 @@
-{- Checks system configuration and generates SysConfig.hs. -}
+{- Checks system configuration and generates SysConfig. -}
+
+{-# OPTIONS_GHC -fno-warn-tabs #-}
 
 module Build.Configure where
 
 import System.Environment
-import Control.Applicative
 import Control.Monad.IfElse
+import Control.Applicative
+import Prelude
 
 import Build.TestConfig
 import Build.Version
@@ -13,7 +16,7 @@ import Git.Version
 tests :: [TestCase]
 tests =
 	[ TestCase "version" (Config "packageversion" . StringConfig <$> getVersion)
-	, TestCase "git" $ requireCmd "git" "git --version >/dev/null"
+	, TestCase "git" $ testCmd "git" "git --version >/dev/null"
 	, TestCase "git version" getGitVersion
 	]
 
@@ -23,7 +26,6 @@ getGitVersion = Config "gitversion" . StringConfig . show
 
 run :: [TestCase] -> IO ()
 run ts = do
-	args <- getArgs
 	config <- runTests ts
 	writeSysConfig config
 	whenM (isReleaseBuild) $
